/*
 * Copyright (c) 1997, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package javax.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.KeyboardFocusManager;
import java.awt.Frame;
import java.awt.Point;
import java.awt.HeadlessException;
import java.awt.Window;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Vector;
import javax.swing.plaf.OptionPaneUI;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameAdapter;
import javax.accessibility.*;
import static javax.swing.ClientPropertyKey.PopupFactory_FORCE_HEAVYWEIGHT_POPUP;
import sun.awt.AWTAccessor;

/**
 * {@code JOptionPane} makes it easy to pop up a standard dialog box that
 * prompts users for a value or informs them of something.
 * For information about using {@code JOptionPane}, see
 * <a
 href="http://docs.oracle.com/javase/tutorial/uiswing/components/dialog.html">How to Make Dialogs</a>,
 * a section in <em>The Java Tutorial</em>.
 *
 * <p>
 *
 * While the {@code JOptionPane}
 * class may appear complex because of the large number of methods, almost
 * all uses of this class are one-line calls to one of the static
 * {@code showXxxDialog} methods shown below:
 * <blockquote>
 *
 *
 * <table border=1 summary="Common JOptionPane method names and their descriptions">
 * <tr>
 *    <th>Method Name</th>
 *    <th>Description</th>
 * </tr>
 * <tr>
 *    <td>showConfirmDialog</td>
 *    <td>Asks a confirming question, like yes/no/cancel.</td>
 * </tr>
 * <tr>
 *    <td>showInputDialog</td>
 *    <td>Prompt for some input.</td>
 * </tr>
 * <tr>
 *   <td>showMessageDialog</td>
 *   <td>Tell the user about something that has happened.</td>
 * </tr>
 * <tr>
 *   <td>showOptionDialog</td>
 *   <td>The Grand Unification of the above three.</td>
 * </tr>
 * </table>
 *
 * </blockquote>
 * Each of these methods also comes in a {@code showInternalXXX}
 * flavor, which uses an internal frame to hold the dialog box (see
 * {@link JInternalFrame}).
 * Multiple convenience methods have also been defined -- overloaded
 * versions of the basic methods that use different parameter lists.
 * <p>
 * All dialogs are modal. Each {@code showXxxDialog} method blocks
 * the caller until the user's interaction is complete.
 *
 * <table cellspacing=6 cellpadding=4 border=0 style="float:right" summary="layout">
 * <tr>
 *  <td style="background-color:#FFe0d0" rowspan=2>icon</td>
 *  <td style="background-color:#FFe0d0">message</td>
 * </tr>
 * <tr>
 *  <td style="background-color:#FFe0d0">input value</td>
 * </tr>
 * <tr>
 *   <td style="background-color:#FFe0d0" colspan=2>option buttons</td>
 * </tr>
 * </table>
 *
 * The basic appearance of one of these dialog boxes is generally
 * similar to the picture at the right, although the various
 * look-and-feels are
 * ultimately responsible for the final result.  In particular, the
 * look-and-feels will adjust the layout to accommodate the option pane's
 * {@code ComponentOrientation} property.
 * <br style="clear:all">
 * <p>
 * <b>Parameters:</b><br>
 * The parameters to these methods follow consistent patterns:
 * <blockquote>
 * <dl>
 * <dt>parentComponent<dd>
 * Defines the {@code Component} that is to be the parent of this
 * dialog box.
 * It is used in two ways: the {@code Frame} that contains
 * it is used as the {@code Frame}
 * parent for the dialog box, and its screen coordinates are used in
 * the placement of the dialog box. In general, the dialog box is placed
 * just below the component. This parameter may be {@code null},
 * in which case a default {@code Frame} is used as the parent,
 * and the dialog will be
 * centered on the screen (depending on the {@literal L&F}).
 * <dt><a name=message>message</a><dd>
 * A descriptive message to be placed in the dialog box.
 * In the most common usage, message is just a {@code String} or
 * {@code String} constant.
 * However, the type of this parameter is actually {@code Object}. Its
 * interpretation depends on its type:
 * <dl>
 * <dt>Object[]<dd>An array of objects is interpreted as a series of
 *                 messages (one per object) arranged in a vertical stack.
 *                 The interpretation is recursive -- each object in the
 *                 array is interpreted according to its type.
 * <dt>Component<dd>The {@code Component} is displayed in the dialog.
 * <dt>Icon<dd>The {@code Icon} is wrapped in a {@code JLabel}
 *               and displayed in the dialog.
 * <dt>others<dd>The object is converted to a {@code String} by calling
 *               its {@code toString} method. The result is wrapped in a
 *               {@code JLabel} and displayed.
 * </dl>
 * <dt>messageType<dd>Defines the style of the message. The Look and Feel
 * manager may lay out the dialog differently depending on this value, and
 * will often provide a default icon. The possible values are:
 * <ul>
 * <li>{@code ERROR_MESSAGE}
 * <li>{@code INFORMATION_MESSAGE}
 * <li>{@code WARNING_MESSAGE}
 * <li>{@code QUESTION_MESSAGE}
 * <li>{@code PLAIN_MESSAGE}
 * </ul>
 * <dt>optionType<dd>Defines the set of option buttons that appear at
 * the bottom of the dialog box:
 * <ul>
 * <li>{@code DEFAULT_OPTION}
 * <li>{@code YES_NO_OPTION}
 * <li>{@code YES_NO_CANCEL_OPTION}
 * <li>{@code OK_CANCEL_OPTION}
 * </ul>
 * You aren't limited to this set of option buttons.  You can provide any
 * buttons you want using the options parameter.
 * <dt>options<dd>A more detailed description of the set of option buttons
 * that will appear at the bottom of the dialog box.
 * The usual value for the options parameter is an array of
 * {@code String}s. But
 * the parameter type is an array of {@code Objects}.
 * A button is created for each object depending on its type:
 * <dl>
 * <dt>Component<dd>The component is added to the button row directly.
 * <dt>Icon<dd>A {@code JButton} is created with this as its label.
 * <dt>other<dd>The {@code Object} is converted to a string using its
 *              {@code toString} method and the result is used to
 *              label a {@code JButton}.
 * </dl>
 * <dt>icon<dd>A decorative icon to be placed in the dialog box. A default
 * value for this is determined by the {@code messageType} parameter.
 * <dt>title<dd>The title for the dialog box.
 * <dt>initialValue<dd>The default selection (input value).
 * </dl>
 * </blockquote>
 * <p>
 * When the selection is changed, {@code setValue} is invoked,
 * which generates a {@code PropertyChangeEvent}.
 * <p>
 * If a {@code JOptionPane} has configured to all input
 * {@code setWantsInput}
 * the bound property {@code JOptionPane.INPUT_VALUE_PROPERTY}
 *  can also be listened
 * to, to determine when the user has input or selected a value.
 * <p>
 * When one of the {@code showXxxDialog} methods returns an integer,
 * the possible values are:
 * <ul>
 * <li>{@code YES_OPTION}
 * <li>{@code NO_OPTION}
 * <li>{@code CANCEL_OPTION}
 * <li>{@code OK_OPTION}
 * <li>{@code CLOSED_OPTION}
 * </ul>
 * <b>Examples:</b>
 * <dl>
 * <dt>Show an error dialog that displays the message, 'alert':
 * <dd>{@code
 * JOptionPane.showMessageDialog(null, "alert", "alert", JOptionPane.ERROR_MESSAGE);
 * }
 * <dt>Show an internal information dialog with the message, 'information':
 * <dd><pre>
 * JOptionPane.showInternalMessageDialog(frame, "information",
 *             "information", JOptionPane.INFORMATION_MESSAGE);
 * </pre>
 * <dt>Show an information panel with the options yes/no and message 'choose one':
 * <dd><pre>JOptionPane.showConfirmDialog(null,
 *             "choose one", "choose one", JOptionPane.YES_NO_OPTION);
 * </pre>
 * <dt>Show an internal information dialog with the options yes/no/cancel and
 * message 'please choose one' and title information:
 * <dd><pre>JOptionPane.showInternalConfirmDialog(frame,
 *             "please choose one", "information",
 *             JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
 * </pre>
 * <dt>Show a warning dialog with the options OK, CANCEL, title 'Warning', and
 * message 'Click OK to continue':
 * <dd><pre>
 * Object[] options = { "OK", "CANCEL" };
 * JOptionPane.showOptionDialog(null, "Click OK to continue", "Warning",
 *             JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
 *             null, options, options[0]);
 * </pre>
 * <dt>Show a dialog asking the user to type in a String:
 * <dd>{@code
 * String inputValue = JOptionPane.showInputDialog("Please input a value");
 * }
 * <dt>Show a dialog asking the user to select a String:
 * <dd><pre>
 * Object[] possibleValues = { "First", "Second", "Third" };<br>
 * Object selectedValue = JOptionPane.showInputDialog(null,
 *             "Choose one", "Input",
 *             JOptionPane.INFORMATION_MESSAGE, null,
 *             possibleValues, possibleValues[0]);
 * </pre>
 * </dl>
 * <b>Direct Use:</b><br>
 * To create and use an {@code JOptionPane} directly, the
 * standard pattern is roughly as follows:
 * <pre>
 *     JOptionPane pane = new JOptionPane(<i>arguments</i>);
 *     pane.set<i>.Xxxx(...); // Configure</i>
 *     JDialog dialog = pane.createDialog(<i>parentComponent, title</i>);
 *     dialog.show();
 *     Object selectedValue = pane.getValue();
 *     if(selectedValue == null)
 *       return CLOSED_OPTION;
 *     <i>//If there is <b>not</b> an array of option buttons:</i>
 *     if(options == null) {
 *       if(selectedValue instanceof Integer)
 *          return ((Integer)selectedValue).intValue();
 *       return CLOSED_OPTION;
 *     }
 *     <i>//If there is an array of option buttons:</i>
 *     for(int counter = 0, maxCounter = options.length;
 *        counter &lt; maxCounter; counter++) {
 *        if(options[counter].equals(selectedValue))
 *        return counter;
 *     }
 *     return CLOSED_OPTION;
 * </pre>
 * <p>
 * <strong>Warning:</strong> Swing is not thread safe. For more
 * information see <a
 * href="package-summary.html#threading">Swing's Threading
 * Policy</a>.
 * <p>
 * <strong>Warning:</strong>
 * Serialized objects of this class will not be compatible with
 * future Swing releases. The current serialization support is
 * appropriate for short term storage or RMI between applications running
 * the same version of Swing.  As of 1.4, support for long term storage
 * of all JavaBeans&trade;
 * has been added to the {@code java.beans} package.
 * Please see {@link java.beans.XMLEncoder}.
 *
 * @see JInternalFrame
 *
 * @beaninfo
 *      attribute: isContainer true
 *    description: A component which implements standard dialog box controls.
 *
 * @author James Gosling
 * @author Scott Violet
 * @since 1.2
 */
@SuppressWarnings("serial") // Same-version serialization only
public class JOptionPane extends JComponent implements Accessible
{
    /**
     * @see #getUIClassID
     * @see #readObject
     */
    private static final String uiClassID = "OptionPaneUI";

    /**
     * Indicates that the user has not yet selected a value.
     */
    public static final Object      UNINITIALIZED_VALUE = "uninitializedValue";

    //
    // Option types
    //

    /**
     * Type meaning Look and Feel should not supply any options -- only
     * use the options from the {@code JOptionPane}.
     */
    public static final int         DEFAULT_OPTION = -1;
    /** Type used for {@code showConfirmDialog}. */
    public static final int         YES_NO_OPTION = 0;
    /** Type used for {@code showConfirmDialog}. */
    public static final int         YES_NO_CANCEL_OPTION = 1;
    /** Type used for {@code showConfirmDialog}. */
    public static final int         OK_CANCEL_OPTION = 2;

    //
    // Return values.
    //
    /** Return value from class method if YES is chosen. */
    public static final int         YES_OPTION = 0;
    /** Return value from class method if NO is chosen. */
    public static final int         NO_OPTION = 1;
    /** Return value from class method if CANCEL is chosen. */
    public static final int         CANCEL_OPTION = 2;
    /** Return value form class method if OK is chosen. */
    public static final int         OK_OPTION = 0;
    /** Return value from class method if user closes window without selecting
     * anything, more than likely this should be treated as either a
     * {@code CANCEL_OPTION} or {@code NO_OPTION}. */
    public static final int         CLOSED_OPTION = -1;

    //
    // Message types. Used by the UI to determine what icon to display,
    // and possibly what behavior to give based on the type.
    //
    /** Used for error messages. */
    public static final int  ERROR_MESSAGE = 0;
    /** Used for information messages. */
    public static final int  INFORMATION_MESSAGE = 1;
    /** Used for warning messages. */
    public static final int  WARNING_MESSAGE = 2;
    /** Used for questions. */
    public static final int  QUESTION_MESSAGE = 3;
    /** No icon is used. */
    public static final int   PLAIN_MESSAGE = -1;

    /** Bound property name for {@code icon}. */
    public static final String      ICON_PROPERTY = "icon";
    /** Bound property name for {@code message}. */
    public static final String      MESSAGE_PROPERTY = "message";
    /** Bound property name for {@code value}. */
    public static final String      VALUE_PROPERTY = "value";
    /** Bound property name for {@code option}. */
    public static final String      OPTIONS_PROPERTY = "options";
    /** Bound property name for {@code initialValue}. */
    public static final String      INITIAL_VALUE_PROPERTY = "initialValue";
    /** Bound property name for {@code type}. */
    public static final String      MESSAGE_TYPE_PROPERTY = "messageType";
    /** Bound property name for {@code optionType}. */
    public static final String      OPTION_TYPE_PROPERTY = "optionType";
    /** Bound property name for {@code selectionValues}. */
    public static final String      SELECTION_VALUES_PROPERTY = "selectionValues";
    /** Bound property name for {@code initialSelectionValue}. */
    public static final String      INITIAL_SELECTION_VALUE_PROPERTY = "initialSelectionValue";
    /** Bound property name for {@code inputValue}. */
    public static final String      INPUT_VALUE_PROPERTY = "inputValue";
    /** Bound property name for {@code wantsInput}. */
    public static final String      WANTS_INPUT_PROPERTY = "wantsInput";

    /** Icon used in pane. */
    protected transient Icon                  icon;
    /** Message to display. */
    protected transient Object                message;
    /** Options to display to the user. */
    protected transient Object[]              options;
    /** Value that should be initially selected in {@code options}. */
    protected transient Object                initialValue;
    /** Message type. */
    protected int                   messageType;
    /**
     * Option type, one of {@code DEFAULT_OPTION},
     * {@code YES_NO_OPTION},
     * {@code YES_NO_CANCEL_OPTION} or
     * {@code OK_CANCEL_OPTION}.
     */
    protected int                   optionType;
    /** Currently selected value, will be a valid option, or
     * {@code UNINITIALIZED_VALUE} or {@code null}. */
    protected transient Object                value;
    /** Array of values the user can choose from. Look and feel will
     * provide the UI component to choose this from. */
    protected transient Object[]              selectionValues;
    /** Value the user has input. */
    protected transient Object                inputValue;
    /** Initial value to select in {@code selectionValues}. */
    protected transient Object                initialSelectionValue;
    /** If true, a UI widget will be provided to the user to get input. */
    protected boolean                         wantsInput;


    /**
     * Shows a question-message dialog requesting input from the user. The
     * dialog uses the default frame, which usually means it is centered on
     * the screen.
     *
     * @param message the {@code Object} to display
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @return user's input
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static String showInputDialog(Object message)
        throws HeadlessException {
        return showInputDialog(null, message);
    }

    /**
     * Shows a question-message dialog requesting input from the user, with
     * the input value initialized to {@code initialSelectionValue}. The
     * dialog uses the default frame, which usually means it is centered on
     * the screen.
     *
     * @param message the {@code Object} to display
     * @param initialSelectionValue the value used to initialize the input
     *                 field
     * @return user's input
     * @since 1.4
     */
    public static String showInputDialog(Object message, Object initialSelectionValue) {
        return showInputDialog(null, message, initialSelectionValue);
    }

    /**
     * Shows a question-message dialog requesting input from the user
     * parented to {@code parentComponent}.
     * The dialog is displayed on top of the {@code Component}'s
     * frame, and is usually positioned below the {@code Component}.
     *
     * @param parentComponent  the parent {@code Component} for the
     *          dialog
     * @param message  the {@code Object} to display
     * @exception HeadlessException if
     *    {@code GraphicsEnvironment.isHeadless} returns
     *    {@code true}
     * @return user's input
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static String showInputDialog(Component parentComponent,
        Object message) throws HeadlessException {
        return showInputDialog(parentComponent, message, UIManager.getString(
            "OptionPane.inputDialogTitle", parentComponent), QUESTION_MESSAGE);
    }

    /**
     * Shows a question-message dialog requesting input from the user and
     * parented to {@code parentComponent}. The input value will be
     * initialized to {@code initialSelectionValue}.
     * The dialog is displayed on top of the {@code Component}'s
     * frame, and is usually positioned below the {@code Component}.
     *
     * @param parentComponent  the parent {@code Component} for the
     *          dialog
     * @param message the {@code Object} to display
     * @param initialSelectionValue the value used to initialize the input
     *                 field
     * @return user's input
     * @since 1.4
     */
    public static String showInputDialog(Component parentComponent, Object message,
                                         Object initialSelectionValue) {
        return (String)showInputDialog(parentComponent, message,
                      UIManager.getString("OptionPane.inputDialogTitle",
                      parentComponent), QUESTION_MESSAGE, null, null,
                      initialSelectionValue);
    }

    /**
     * Shows a dialog requesting input from the user parented to
     * {@code parentComponent} with the dialog having the title
     * {@code title} and message type {@code messageType}.
     *
     * @param parentComponent  the parent {@code Component} for the
     *                  dialog
     * @param message  the {@code Object} to display
     * @param title    the {@code String} to display in the dialog
     *                  title bar
     * @param messageType the type of message that is to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @return user's input
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static String showInputDialog(Component parentComponent,
        Object message, String title, int messageType)
        throws HeadlessException {
        return (String)showInputDialog(parentComponent, message, title,
                                       messageType, null, null, null);
    }

    /**
     * Prompts the user for input in a blocking dialog where the
     * initial selection, possible selections, and all other options can
     * be specified. The user will able to choose from
     * {@code selectionValues}, where {@code null} implies the
     * user can input
     * whatever they wish, usually by means of a {@code JTextField}.
     * {@code initialSelectionValue} is the initial value to prompt
     * the user with. It is up to the UI to decide how best to represent
     * the {@code selectionValues}, but usually a
     * {@code JComboBox}, {@code JList}, or
     * {@code JTextField} will be used.
     *
     * @param parentComponent  the parent {@code Component} for the
     *                  dialog
     * @param message  the {@code Object} to display
     * @param title    the {@code String} to display in the
     *                  dialog title bar
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param icon     the {@code Icon} image to display
     * @param selectionValues an array of {@code Object}s that
     *                  gives the possible selections
     * @param initialSelectionValue the value used to initialize the input
     *                 field
     * @return user's input, or {@code null} meaning the user
     *                  canceled the input
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    @SuppressWarnings("deprecation")
    public static Object showInputDialog(Component parentComponent,
        Object message, String title, int messageType, Icon icon,
        Object[] selectionValues, Object initialSelectionValue)
        throws HeadlessException {
        JOptionPane    pane = new JOptionPane(message, messageType,
                                              OK_CANCEL_OPTION, icon,
                                              null, null);

        pane.setWantsInput(true);
        pane.setSelectionValues(selectionValues);
        pane.setInitialSelectionValue(initialSelectionValue);
        pane.setComponentOrientation(((parentComponent == null) ?
            getRootFrame() : parentComponent).getComponentOrientation());

        int style = styleFromMessageType(messageType);
        JDialog dialog = pane.createDialog(parentComponent, title, style);

        pane.selectInitialValue();
        dialog.show();
        dialog.dispose();

        Object value = pane.getInputValue();

        if (value == UNINITIALIZED_VALUE) {
            return null;
        }
        return value;
    }

    /**
     * Brings up an information-message dialog titled "Message".
     *
     * @param parentComponent determines the {@code Frame} in
     *          which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static void showMessageDialog(Component parentComponent,
        Object message) throws HeadlessException {
        showMessageDialog(parentComponent, message, UIManager.getString(
                    "OptionPane.messageDialogTitle", parentComponent),
                    INFORMATION_MESSAGE);
    }

    /**
     * Brings up a dialog that displays a message using a default
     * icon determined by the {@code messageType} parameter.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static void showMessageDialog(Component parentComponent,
        Object message, String title, int messageType)
        throws HeadlessException {
        showMessageDialog(parentComponent, message, title, messageType, null);
    }

    /**
     * Brings up a dialog displaying a message, specifying all parameters.
     *
     * @param parentComponent determines the {@code Frame} in which the
     *                  dialog is displayed; if {@code null},
     *                  or if the {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param icon      an icon to display in the dialog that helps the user
     *                  identify the kind of message that is being displayed
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static void showMessageDialog(Component parentComponent,
        Object message, String title, int messageType, Icon icon)
        throws HeadlessException {
        showOptionDialog(parentComponent, message, title, DEFAULT_OPTION,
                         messageType, icon, null, null);
    }

    /**
     * Brings up a dialog with the options <i>Yes</i>,
     * <i>No</i> and <i>Cancel</i>; with the
     * title, <b>Select an Option</b>.
     *
     * @param parentComponent determines the {@code Frame} in which the
     *                  dialog is displayed; if {@code null},
     *                  or if the {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @return an integer indicating the option selected by the user
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static int showConfirmDialog(Component parentComponent,
        Object message) throws HeadlessException {
        return showConfirmDialog(parentComponent, message,
                                 UIManager.getString("OptionPane.titleText"),
                                 YES_NO_CANCEL_OPTION);
    }

    /**
     * Brings up a dialog where the number of choices is determined
     * by the {@code optionType} parameter.
     *
     * @param parentComponent determines the {@code Frame} in which the
     *                  dialog is displayed; if {@code null},
     *                  or if the {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param optionType an int designating the options available on the dialog:
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  or {@code OK_CANCEL_OPTION}
     * @return an int indicating the option selected by the user
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static int showConfirmDialog(Component parentComponent,
        Object message, String title, int optionType)
        throws HeadlessException {
        return showConfirmDialog(parentComponent, message, title, optionType,
                                 QUESTION_MESSAGE);
    }

    /**
     * Brings up a dialog where the number of choices is determined
     * by the {@code optionType} parameter, where the
     * {@code messageType}
     * parameter determines the icon to display.
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the Look and Feel.
     *
     * @param parentComponent determines the {@code Frame} in
     *                  which the dialog is displayed; if {@code null},
     *                  or if the {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used.
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options available
     *                   on the dialog: {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  or {@code OK_CANCEL_OPTION}
     * @param messageType an integer designating the kind of message this is;
     *                  primarily used to determine the icon from the pluggable
     *                  Look and Feel: {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @return an integer indicating the option selected by the user
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static int showConfirmDialog(Component parentComponent,
        Object message, String title, int optionType, int messageType)
        throws HeadlessException {
        return showConfirmDialog(parentComponent, message, title, optionType,
                                messageType, null);
    }

    /**
     * Brings up a dialog with a specified icon, where the number of
     * choices is determined by the {@code optionType} parameter.
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the look and feel.
     *
     * @param parentComponent determines the {@code Frame} in which the
     *                  dialog is displayed; if {@code null},
     *                  or if the {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used
     * @param message   the Object to display
     * @param title     the title string for the dialog
     * @param optionType an int designating the options available on the dialog:
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  or {@code OK_CANCEL_OPTION}
     * @param messageType an int designating the kind of message this is,
     *                  primarily used to determine the icon from the pluggable
     *                  Look and Feel: {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param icon      the icon to display in the dialog
     * @return an int indicating the option selected by the user
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static int showConfirmDialog(Component parentComponent,
        Object message, String title, int optionType,
        int messageType, Icon icon) throws HeadlessException {
        return showOptionDialog(parentComponent, message, title, optionType,
                                messageType, icon, null, null);
    }

    /**
     * Brings up a dialog with a specified icon, where the initial
     * choice is determined by the {@code initialValue} parameter and
     * the number of choices is determined by the {@code optionType}
     * parameter.
     * <p>
     * If {@code optionType} is {@code YES_NO_OPTION},
     * or {@code YES_NO_CANCEL_OPTION}
     * and the {@code options} parameter is {@code null},
     * then the options are
     * supplied by the look and feel.
     * <p>
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the look and feel.
     *
     * @param parentComponent determines the {@code Frame}
     *                  in which the dialog is displayed;  if
     *                  {@code null}, or if the
     *                  {@code parentComponent} has no
     *                  {@code Frame}, a
     *                  default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options available on the
     *                  dialog: {@code DEFAULT_OPTION},
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  or {@code OK_CANCEL_OPTION}
     * @param messageType an integer designating the kind of message this is,
     *                  primarily used to determine the icon from the
     *                  pluggable Look and Feel: {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param icon      the icon to display in the dialog
     * @param options   an array of objects indicating the possible choices
     *                  the user can make; if the objects are components, they
     *                  are rendered properly; non-{@code String}
     *                  objects are
     *                  rendered using their {@code toString} methods;
     *                  if this parameter is {@code null},
     *                  the options are determined by the Look and Feel
     * @param initialValue the object that represents the default selection
     *                  for the dialog; only meaningful if {@code options}
     *                  is used; can be {@code null}
     * @return an integer indicating the option chosen by the user,
     *                  or {@code CLOSED_OPTION} if the user closed
     *                  the dialog
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    @SuppressWarnings("deprecation")
    public static int showOptionDialog(Component parentComponent,
        Object message, String title, int optionType, int messageType,
        Icon icon, Object[] options, Object initialValue)
        throws HeadlessException {
        JOptionPane             pane = new JOptionPane(message, messageType,
                                                       optionType, icon,
                                                       options, initialValue);

        pane.setInitialValue(initialValue);
        pane.setComponentOrientation(((parentComponent == null) ?
            getRootFrame() : parentComponent).getComponentOrientation());

        int style = styleFromMessageType(messageType);
        JDialog dialog = pane.createDialog(parentComponent, title, style);

        pane.selectInitialValue();
        dialog.show();
        dialog.dispose();

        Object        selectedValue = pane.getValue();

        if(selectedValue == null)
            return CLOSED_OPTION;
        if(options == null) {
            if(selectedValue instanceof Integer)
                return ((Integer)selectedValue).intValue();
            return CLOSED_OPTION;
        }
        for(int counter = 0, maxCounter = options.length;
            counter < maxCounter; counter++) {
            if(options[counter].equals(selectedValue))
                return counter;
        }
        return CLOSED_OPTION;
    }

    /**
     * Creates and returns a new {@code JDialog} wrapping
     * {@code this} centered on the {@code parentComponent}
     * in the {@code parentComponent}'s frame.
     * {@code title} is the title of the returned dialog.
     * The returned {@code JDialog} will not be resizable by the
     * user, however programs can invoke {@code setResizable} on
     * the {@code JDialog} instance to change this property.
     * The returned {@code JDialog} will be set up such that
     * once it is closed, or the user clicks on one of the buttons,
     * the optionpane's value property will be set accordingly and
     * the dialog will be closed.  Each time the dialog is made visible,
     * it will reset the option pane's value property to
     * {@code JOptionPane.UNINITIALIZED_VALUE} to ensure the
     * user's subsequent action closes the dialog properly.
     *
     * @param parentComponent determines the frame in which the dialog
     *          is displayed; if the {@code parentComponent} has
     *          no {@code Frame}, a default {@code Frame} is used
     * @param title     the title string for the dialog
     * @return a new {@code JDialog} containing this instance
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public JDialog createDialog(Component parentComponent, String title)
        throws HeadlessException {
        int style = styleFromMessageType(getMessageType());
        return createDialog(parentComponent, title, style);
    }

    /**
     * Creates and returns a new parentless {@code JDialog}
     * with the specified title.
     * The returned {@code JDialog} will not be resizable by the
     * user, however programs can invoke {@code setResizable} on
     * the {@code JDialog} instance to change this property.
     * The returned {@code JDialog} will be set up such that
     * once it is closed, or the user clicks on one of the buttons,
     * the optionpane's value property will be set accordingly and
     * the dialog will be closed.  Each time the dialog is made visible,
     * it will reset the option pane's value property to
     * {@code JOptionPane.UNINITIALIZED_VALUE} to ensure the
     * user's subsequent action closes the dialog properly.
     *
     * @param title     the title string for the dialog
     * @return a new {@code JDialog} containing this instance
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     * @since 1.6
     */
    public JDialog createDialog(String title) throws HeadlessException {
        int style = styleFromMessageType(getMessageType());
        JDialog dialog = new JDialog((Dialog) null, title, true);
        initDialog(dialog, style, null);
        return dialog;
    }

    private JDialog createDialog(Component parentComponent, String title,
            int style)
            throws HeadlessException {

        final JDialog dialog;

        Window window = JOptionPane.getWindowForComponent(parentComponent);
        if (window instanceof Frame) {
            dialog = new JDialog((Frame)window, title, true);
        } else {
            dialog = new JDialog((Dialog)window, title, true);
        }
        if (window instanceof SwingUtilities.SharedOwnerFrame) {
            WindowListener ownerShutdownListener =
                    SwingUtilities.getSharedOwnerFrameShutdownListener();
            dialog.addWindowListener(ownerShutdownListener);
        }
        initDialog(dialog, style, parentComponent);
        return dialog;
    }

    private void initDialog(final JDialog dialog, int style, Component parentComponent) {
        dialog.setComponentOrientation(this.getComponentOrientation());
        Container contentPane = dialog.getContentPane();

        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);
        dialog.setResizable(false);
        if (JDialog.isDefaultLookAndFeelDecorated()) {
            boolean supportsWindowDecorations =
              UIManager.getLookAndFeel().getSupportsWindowDecorations();
            if (supportsWindowDecorations) {
                dialog.setUndecorated(true);
                getRootPane().setWindowDecorationStyle(style);
            }
        }
        dialog.pack();
        dialog.setLocationRelativeTo(parentComponent);

        final PropertyChangeListener listener = new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent event) {
                // Let the defaultCloseOperation handle the closing
                // if the user closed the window without selecting a button
                // (newValue = null in that case).  Otherwise, close the dialog.
                if (dialog.isVisible() && event.getSource() == JOptionPane.this &&
                        (event.getPropertyName().equals(VALUE_PROPERTY)) &&
                        event.getNewValue() != null &&
                        event.getNewValue() != JOptionPane.UNINITIALIZED_VALUE) {
                    dialog.setVisible(false);
                }
            }
        };

        WindowAdapter adapter = new WindowAdapter() {
            private boolean gotFocus = false;
            public void windowClosing(WindowEvent we) {
                setValue(null);
            }

            public void windowClosed(WindowEvent e) {
                removePropertyChangeListener(listener);
                dialog.getContentPane().removeAll();
            }

            public void windowGainedFocus(WindowEvent we) {
                // Once window gets focus, set initial focus
                if (!gotFocus) {
                    selectInitialValue();
                    gotFocus = true;
                }
            }
        };
        dialog.addWindowListener(adapter);
        dialog.addWindowFocusListener(adapter);
        dialog.addComponentListener(new ComponentAdapter() {
            public void componentShown(ComponentEvent ce) {
                // reset value to ensure closing works properly
                setValue(JOptionPane.UNINITIALIZED_VALUE);
            }
        });

        addPropertyChangeListener(listener);
    }


    /**
     * Brings up an internal confirmation dialog panel. The dialog
     * is a information-message dialog titled "Message".
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the object to display
     */
    public static void showInternalMessageDialog(Component parentComponent,
                                                 Object message) {
        showInternalMessageDialog(parentComponent, message, UIManager.
                                 getString("OptionPane.messageDialogTitle",
                                 parentComponent), INFORMATION_MESSAGE);
    }

    /**
     * Brings up an internal dialog panel that displays a message
     * using a default icon determined by the {@code messageType}
     * parameter.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     */
    public static void showInternalMessageDialog(Component parentComponent,
                                                 Object message, String title,
                                                 int messageType) {
        showInternalMessageDialog(parentComponent, message, title, messageType,null);
    }

    /**
     * Brings up an internal dialog panel displaying a message,
     * specifying all parameters.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @param title     the title string for the dialog
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param icon      an icon to display in the dialog that helps the user
     *                  identify the kind of message that is being displayed
     */
    public static void showInternalMessageDialog(Component parentComponent,
                                         Object message,
                                         String title, int messageType,
                                         Icon icon){
        showInternalOptionDialog(parentComponent, message, title, DEFAULT_OPTION,
                                 messageType, icon, null, null);
    }

    /**
     * Brings up an internal dialog panel with the options <i>Yes</i>, <i>No</i>
     * and <i>Cancel</i>; with the title, <b>Select an Option</b>.
     *
     * @param parentComponent determines the {@code Frame} in
     *          which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the {@code Object} to display
     * @return an integer indicating the option selected by the user
     */
    public static int showInternalConfirmDialog(Component parentComponent,
                                                Object message) {
        return showInternalConfirmDialog(parentComponent, message,
                                 UIManager.getString("OptionPane.titleText"),
                                 YES_NO_CANCEL_OPTION);
    }

    /**
     * Brings up a internal dialog panel where the number of choices
     * is determined by the {@code optionType} parameter.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the object to display in the dialog; a
     *          {@code Component} object is rendered as a
     *          {@code Component}; a {@code String}
     *          object is rendered as a string; other objects
     *          are converted to a {@code String} using the
     *          {@code toString} method
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options
     *          available on the dialog: {@code YES_NO_OPTION},
     *          or {@code YES_NO_CANCEL_OPTION}
     * @return an integer indicating the option selected by the user
     */
    public static int showInternalConfirmDialog(Component parentComponent,
                                                Object message, String title,
                                                int optionType) {
        return showInternalConfirmDialog(parentComponent, message, title, optionType,
                                         QUESTION_MESSAGE);
    }

    /**
     * Brings up an internal dialog panel where the number of choices
     * is determined by the {@code optionType} parameter, where
     * the {@code messageType} parameter determines the icon to display.
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the Look and Feel.
     *
     * @param parentComponent determines the {@code Frame} in
     *          which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the object to display in the dialog; a
     *          {@code Component} object is rendered as a
     *          {@code Component}; a {@code String}
     *          object is rendered as a string; other objects are
     *          converted to a {@code String} using the
     *          {@code toString} method
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options
     *          available on the dialog:
     *          {@code YES_NO_OPTION}, or {@code YES_NO_CANCEL_OPTION}
     * @param messageType an integer designating the kind of message this is,
     *          primarily used to determine the icon from the
     *          pluggable Look and Feel: {@code ERROR_MESSAGE},
     *          {@code INFORMATION_MESSAGE},
     *          {@code WARNING_MESSAGE}, {@code QUESTION_MESSAGE},
     *          or {@code PLAIN_MESSAGE}
     * @return an integer indicating the option selected by the user
     */
    public static int showInternalConfirmDialog(Component parentComponent,
                                        Object message,
                                        String title, int optionType,
                                        int messageType) {
        return showInternalConfirmDialog(parentComponent, message, title, optionType,
                                         messageType, null);
    }

    /**
     * Brings up an internal dialog panel with a specified icon, where
     * the number of choices is determined by the {@code optionType}
     * parameter.
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the look and feel.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the parentComponent has no Frame, a
     *          default {@code Frame} is used
     * @param message   the object to display in the dialog; a
     *          {@code Component} object is rendered as a
     *          {@code Component}; a {@code String}
     *          object is rendered as a string; other objects are
     *          converted to a {@code String} using the
     *          {@code toString} method
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options available
     *          on the dialog:
     *          {@code YES_NO_OPTION}, or
     *          {@code YES_NO_CANCEL_OPTION}.
     * @param messageType an integer designating the kind of message this is,
     *          primarily used to determine the icon from the pluggable
     *          Look and Feel: {@code ERROR_MESSAGE},
     *          {@code INFORMATION_MESSAGE},
     *          {@code WARNING_MESSAGE}, {@code QUESTION_MESSAGE},
     *          or {@code PLAIN_MESSAGE}
     * @param icon      the icon to display in the dialog
     * @return an integer indicating the option selected by the user
     */
    public static int showInternalConfirmDialog(Component parentComponent,
                                        Object message,
                                        String title, int optionType,
                                        int messageType, Icon icon) {
        return showInternalOptionDialog(parentComponent, message, title, optionType,
                                        messageType, icon, null, null);
    }

    /**
     * Brings up an internal dialog panel with a specified icon, where
     * the initial choice is determined by the {@code initialValue}
     * parameter and the number of choices is determined by the
     * {@code optionType} parameter.
     * <p>
     * If {@code optionType} is {@code YES_NO_OPTION}, or
     * {@code YES_NO_CANCEL_OPTION}
     * and the {@code options} parameter is {@code null},
     * then the options are supplied by the Look and Feel.
     * <p>
     * The {@code messageType} parameter is primarily used to supply
     * a default icon from the look and feel.
     *
     * @param parentComponent determines the {@code Frame}
     *          in which the dialog is displayed; if {@code null},
     *          or if the {@code parentComponent} has no
     *          {@code Frame}, a default {@code Frame} is used
     * @param message   the object to display in the dialog; a
     *          {@code Component} object is rendered as a
     *          {@code Component}; a {@code String}
     *          object is rendered as a string. Other objects are
     *          converted to a {@code String} using the
     *          {@code toString} method
     * @param title     the title string for the dialog
     * @param optionType an integer designating the options available
     *          on the dialog: {@code YES_NO_OPTION},
     *          or {@code YES_NO_CANCEL_OPTION}
     * @param messageType an integer designating the kind of message this is;
     *          primarily used to determine the icon from the
     *          pluggable Look and Feel: {@code ERROR_MESSAGE},
     *          {@code INFORMATION_MESSAGE},
     *          {@code WARNING_MESSAGE}, {@code QUESTION_MESSAGE},
     *          or {@code PLAIN_MESSAGE}
     * @param icon      the icon to display in the dialog
     * @param options   an array of objects indicating the possible choices
     *          the user can make; if the objects are components, they
     *          are rendered properly; non-{@code String}
     *          objects are rendered using their {@code toString}
     *          methods; if this parameter is {@code null},
     *          the options are determined by the Look and Feel
     * @param initialValue the object that represents the default selection
     *          for the dialog; only meaningful if {@code options}
     *          is used; can be {@code null}
     * @return an integer indicating the option chosen by the user,
     *          or {@code CLOSED_OPTION} if the user closed the Dialog
     */
    public static int showInternalOptionDialog(Component parentComponent,
                                       Object message,
                                       String title, int optionType,
                                       int messageType, Icon icon,
                                       Object[] options, Object initialValue) {
        JOptionPane pane = new JOptionPane(message, messageType,
                optionType, icon, options, initialValue);
        pane.putClientProperty(PopupFactory_FORCE_HEAVYWEIGHT_POPUP,
                Boolean.TRUE);
        Component fo = KeyboardFocusManager.getCurrentKeyboardFocusManager().
                getFocusOwner();

        pane.setInitialValue(initialValue);

        JInternalFrame dialog =
            pane.createInternalFrame(parentComponent, title);
        pane.selectInitialValue();
        dialog.setVisible(true);

        /* Since all input will be blocked until this dialog is dismissed,
         * make sure its parent containers are visible first (this component
         * is tested below).  This is necessary for JApplets, because
         * because an applet normally isn't made visible until after its
         * start() method returns -- if this method is called from start(),
         * the applet will appear to hang while an invisible modal frame
         * waits for input.
         */
        if (dialog.isVisible() && !dialog.isShowing()) {
            Container parent = dialog.getParent();
            while (parent != null) {
                if (parent.isVisible() == false) {
                    parent.setVisible(true);
                }
                parent = parent.getParent();
            }
        }

        AWTAccessor.getContainerAccessor().startLWModal(dialog);

        if (parentComponent instanceof JInternalFrame) {
            try {
                ((JInternalFrame)parentComponent).setSelected(true);
            } catch (java.beans.PropertyVetoException e) {
            }
        }

        Object selectedValue = pane.getValue();

        if (fo != null && fo.isShowing()) {
            fo.requestFocus();
        }
        if (selectedValue == null) {
            return CLOSED_OPTION;
        }
        if (options == null) {
            if (selectedValue instanceof Integer) {
                return ((Integer)selectedValue).intValue();
            }
            return CLOSED_OPTION;
        }
        for(int counter = 0, maxCounter = options.length;
            counter < maxCounter; counter++) {
            if (options[counter].equals(selectedValue)) {
                return counter;
            }
        }
        return CLOSED_OPTION;
    }

    /**
     * Shows an internal question-message dialog requesting input from
     * the user parented to {@code parentComponent}. The dialog
     * is displayed in the {@code Component}'s frame,
     * and is usually positioned below the {@code Component}.
     *
     * @param parentComponent  the parent {@code Component}
     *          for the dialog
     * @param message  the {@code Object} to display
     * @return user's input
     */
    public static String showInternalInputDialog(Component parentComponent,
                                                 Object message) {
        return showInternalInputDialog(parentComponent, message, UIManager.
               getString("OptionPane.inputDialogTitle", parentComponent),
               QUESTION_MESSAGE);
    }

    /**
     * Shows an internal dialog requesting input from the user parented
     * to {@code parentComponent} with the dialog having the title
     * {@code title} and message type {@code messageType}.
     *
     * @param parentComponent the parent {@code Component} for the dialog
     * @param message  the {@code Object} to display
     * @param title    the {@code String} to display in the
     *          dialog title bar
     * @param messageType the type of message that is to be displayed:
     *                    ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                    QUESTION_MESSAGE, or PLAIN_MESSAGE
     * @return user's input
     */
    public static String showInternalInputDialog(Component parentComponent,
                             Object message, String title, int messageType) {
        return (String)showInternalInputDialog(parentComponent, message, title,
                                       messageType, null, null, null);
    }

    /**
     * Prompts the user for input in a blocking internal dialog where
     * the initial selection, possible selections, and all other
     * options can be specified. The user will able to choose from
     * {@code selectionValues}, where {@code null}
     * implies the user can input
     * whatever they wish, usually by means of a {@code JTextField}.
     * {@code initialSelectionValue} is the initial value to prompt
     * the user with. It is up to the UI to decide how best to represent
     * the {@code selectionValues}, but usually a
     * {@code JComboBox}, {@code JList}, or
     * {@code JTextField} will be used.
     *
     * @param parentComponent the parent {@code Component} for the dialog
     * @param message  the {@code Object} to display
     * @param title    the {@code String} to display in the dialog
     *          title bar
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE}, {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE}, or {@code PLAIN_MESSAGE}
     * @param icon     the {@code Icon} image to display
     * @param selectionValues an array of {@code Objects} that
     *                  gives the possible selections
     * @param initialSelectionValue the value used to initialize the input
     *                  field
     * @return user's input, or {@code null} meaning the user
     *          canceled the input
     */
    public static Object showInternalInputDialog(Component parentComponent,
            Object message, String title, int messageType, Icon icon,
            Object[] selectionValues, Object initialSelectionValue) {
        JOptionPane pane = new JOptionPane(message, messageType,
                OK_CANCEL_OPTION, icon, null, null);
        pane.putClientProperty(PopupFactory_FORCE_HEAVYWEIGHT_POPUP,
                Boolean.TRUE);
        Component fo = KeyboardFocusManager.getCurrentKeyboardFocusManager().
                getFocusOwner();

        pane.setWantsInput(true);
        pane.setSelectionValues(selectionValues);
        pane.setInitialSelectionValue(initialSelectionValue);

        JInternalFrame dialog =
            pane.createInternalFrame(parentComponent, title);

        pane.selectInitialValue();
        dialog.setVisible(true);

        /* Since all input will be blocked until this dialog is dismissed,
         * make sure its parent containers are visible first (this component
         * is tested below).  This is necessary for JApplets, because
         * because an applet normally isn't made visible until after its
         * start() method returns -- if this method is called from start(),
         * the applet will appear to hang while an invisible modal frame
         * waits for input.
         */
        if (dialog.isVisible() && !dialog.isShowing()) {
            Container parent = dialog.getParent();
            while (parent != null) {
                if (parent.isVisible() == false) {
                    parent.setVisible(true);
                }
                parent = parent.getParent();
            }
        }

        AWTAccessor.getContainerAccessor().startLWModal(dialog);

        if (parentComponent instanceof JInternalFrame) {
            try {
                ((JInternalFrame)parentComponent).setSelected(true);
            } catch (java.beans.PropertyVetoException e) {
            }
        }

        if (fo != null && fo.isShowing()) {
            fo.requestFocus();
        }
        Object value = pane.getInputValue();

        if (value == UNINITIALIZED_VALUE) {
            return null;
        }
        return value;
    }

    /**
     * Creates and returns an instance of {@code JInternalFrame}.
     * The internal frame is created with the specified title,
     * and wrapping the {@code JOptionPane}.
     * The returned {@code JInternalFrame} is
     * added to the {@code JDesktopPane} ancestor of
     * {@code parentComponent}, or components
     * parent if one its ancestors isn't a {@code JDesktopPane},
     * or if {@code parentComponent}
     * doesn't have a parent then a {@code RuntimeException} is thrown.
     *
     * @param parentComponent  the parent {@code Component} for
     *          the internal frame
     * @param title    the {@code String} to display in the
     *          frame's title bar
     * @return a {@code JInternalFrame} containing a
     *          {@code JOptionPane}
     * @exception RuntimeException if {@code parentComponent} does
     *          not have a valid parent
     */
    public JInternalFrame createInternalFrame(Component parentComponent,
                                 String title) {
        Container parent =
                JOptionPane.getDesktopPaneForComponent(parentComponent);

        if (parent == null && (parentComponent == null ||
                (parent = parentComponent.getParent()) == null)) {
            throw new RuntimeException("JOptionPane: parentComponent does " +
                    "not have a valid parent");
        }

        // Option dialogs should be closable only
        final JInternalFrame  iFrame = new JInternalFrame(title, false, true,
                                                           false, false);

        iFrame.putClientProperty("JInternalFrame.frameType", "optionDialog");
        iFrame.putClientProperty("JInternalFrame.messageType",
                                 Integer.valueOf(getMessageType()));

        iFrame.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(InternalFrameEvent e) {
                if (getValue() == UNINITIALIZED_VALUE) {
                    setValue(null);
                }
            }
        });
        addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent event) {
                // Let the defaultCloseOperation handle the closing
                // if the user closed the iframe without selecting a button
                // (newValue = null in that case).  Otherwise, close the dialog.
                if (iFrame.isVisible() &&
                        event.getSource() == JOptionPane.this &&
                        event.getPropertyName().equals(VALUE_PROPERTY)) {
                    AWTAccessor.getContainerAccessor().stopLWModal(iFrame);

                try {
                    iFrame.setClosed(true);
                }
                catch (java.beans.PropertyVetoException e) {
                }

                iFrame.setVisible(false);
                }
            }
        });
        iFrame.getContentPane().add(this, BorderLayout.CENTER);
        if (parent instanceof JDesktopPane) {
            parent.add(iFrame, JLayeredPane.MODAL_LAYER);
        } else {
            parent.add(iFrame, BorderLayout.CENTER);
        }
        Dimension iFrameSize = iFrame.getPreferredSize();
        Dimension rootSize = parent.getSize();
        Dimension parentSize = parentComponent.getSize();

        iFrame.setBounds((rootSize.width - iFrameSize.width) / 2,
                         (rootSize.height - iFrameSize.height) / 2,
                         iFrameSize.width, iFrameSize.height);
        // We want dialog centered relative to its parent component
        Point iFrameCoord =
          SwingUtilities.convertPoint(parentComponent, 0, 0, parent);
        int x = (parentSize.width - iFrameSize.width) / 2 + iFrameCoord.x;
        int y = (parentSize.height - iFrameSize.height) / 2 + iFrameCoord.y;

        // If possible, dialog should be fully visible
        int ovrx = x + iFrameSize.width - rootSize.width;
        int ovry = y + iFrameSize.height - rootSize.height;
        x = Math.max((ovrx > 0? x - ovrx: x), 0);
        y = Math.max((ovry > 0? y - ovry: y), 0);
        iFrame.setBounds(x, y, iFrameSize.width, iFrameSize.height);

        parent.validate();
        try {
            iFrame.setSelected(true);
        } catch (java.beans.PropertyVetoException e) {}

        return iFrame;
    }

    /**
     * Returns the specified component's {@code Frame}.
     *
     * @param parentComponent the {@code Component} to check for a
     *          {@code Frame}
     * @return the {@code Frame} that contains the component,
     *          or {@code getRootFrame}
     *          if the component is {@code null},
     *          or does not have a valid {@code Frame} parent
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see #getRootFrame
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static Frame getFrameForComponent(Component parentComponent)
        throws HeadlessException {
        if (parentComponent == null)
            return getRootFrame();
        if (parentComponent instanceof Frame)
            return (Frame)parentComponent;
        return JOptionPane.getFrameForComponent(parentComponent.getParent());
    }

    /**
     * Returns the specified component's toplevel {@code Frame} or
     * {@code Dialog}.
     *
     * @param parentComponent the {@code Component} to check for a
     *          {@code Frame} or {@code Dialog}
     * @return the {@code Frame} or {@code Dialog} that
     *          contains the component, or the default
     *          frame if the component is {@code null},
     *          or does not have a valid
     *          {@code Frame} or {@code Dialog} parent
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    static Window getWindowForComponent(Component parentComponent)
        throws HeadlessException {
        if (parentComponent == null)
            return getRootFrame();
        if (parentComponent instanceof Frame || parentComponent instanceof Dialog)
            return (Window)parentComponent;
        return JOptionPane.getWindowForComponent(parentComponent.getParent());
    }


    /**
     * Returns the specified component's desktop pane.
     *
     * @param parentComponent the {@code Component} to check for a
     *          desktop
     * @return the {@code JDesktopPane} that contains the component,
     *          or {@code null} if the component is {@code null}
     *          or does not have an ancestor that is a
     *          {@code JInternalFrame}
     */
    public static JDesktopPane getDesktopPaneForComponent(Component parentComponent) {
        if(parentComponent == null)
            return null;
        if(parentComponent instanceof JDesktopPane)
            return (JDesktopPane)parentComponent;
        return getDesktopPaneForComponent(parentComponent.getParent());
    }

    private static final Object sharedFrameKey = JOptionPane.class;

    /**
     * Sets the frame to use for class methods in which a frame is
     * not provided.
     * <p>
     * <strong>Note:</strong>
     * It is recommended that rather than using this method you supply a valid parent.
     *
     * @param newRootFrame the default {@code Frame} to use
     */
    public static void setRootFrame(Frame newRootFrame) {
        if (newRootFrame != null) {
            SwingUtilities.appContextPut(sharedFrameKey, newRootFrame);
        } else {
            SwingUtilities.appContextRemove(sharedFrameKey);
        }
    }

    /**
     * Returns the {@code Frame} to use for the class methods in
     * which a frame is not provided.
     *
     * @return the default {@code Frame} to use
     * @exception HeadlessException if
     *   {@code GraphicsEnvironment.isHeadless} returns
     *   {@code true}
     * @see #setRootFrame
     * @see java.awt.GraphicsEnvironment#isHeadless
     */
    public static Frame getRootFrame() throws HeadlessException {
        Frame sharedFrame =
            (Frame)SwingUtilities.appContextGet(sharedFrameKey);
        if (sharedFrame == null) {
            sharedFrame = SwingUtilities.getSharedOwnerFrame();
            SwingUtilities.appContextPut(sharedFrameKey, sharedFrame);
        }
        return sharedFrame;
    }

    /**
     * Creates a {@code JOptionPane} with a test message.
     */
    public JOptionPane() {
        this("JOptionPane message");
    }

    /**
     * Creates a instance of {@code JOptionPane} to display a
     * message using the
     * plain-message message type and the default options delivered by
     * the UI.
     *
     * @param message the {@code Object} to display
     */
    public JOptionPane(Object message) {
        this(message, PLAIN_MESSAGE);
    }

    /**
     * Creates an instance of {@code JOptionPane} to display a message
     * with the specified message type and the default options,
     *
     * @param message the {@code Object} to display
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     */
    public JOptionPane(Object message, int messageType) {
        this(message, messageType, DEFAULT_OPTION);
    }

    /**
     * Creates an instance of {@code JOptionPane} to display a message
     * with the specified message type and options.
     *
     * @param message the {@code Object} to display
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param optionType the options to display in the pane:
     *                  {@code DEFAULT_OPTION}, {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  {@code OK_CANCEL_OPTION}
     */
    public JOptionPane(Object message, int messageType, int optionType) {
        this(message, messageType, optionType, null);
    }

    /**
     * Creates an instance of {@code JOptionPane} to display a message
     * with the specified message type, options, and icon.
     *
     * @param message the {@code Object} to display
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param optionType the options to display in the pane:
     *                  {@code DEFAULT_OPTION}, {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  {@code OK_CANCEL_OPTION}
     * @param icon the {@code Icon} image to display
     */
    public JOptionPane(Object message, int messageType, int optionType,
                       Icon icon) {
        this(message, messageType, optionType, icon, null);
    }

    /**
     * Creates an instance of {@code JOptionPane} to display a message
     * with the specified message type, icon, and options.
     * None of the options is initially selected.
     * <p>
     * The options objects should contain either instances of
     * {@code Component}s, (which are added directly) or
     * {@code Strings} (which are wrapped in a {@code JButton}).
     * If you provide {@code Component}s, you must ensure that when the
     * {@code Component} is clicked it messages {@code setValue}
     * in the created {@code JOptionPane}.
     *
     * @param message the {@code Object} to display
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param optionType the options to display in the pane:
     *                  {@code DEFAULT_OPTION},
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  {@code OK_CANCEL_OPTION}
     * @param icon the {@code Icon} image to display
     * @param options  the choices the user can select
     */
    public JOptionPane(Object message, int messageType, int optionType,
                       Icon icon, Object[] options) {
        this(message, messageType, optionType, icon, options, null);
    }

    /**
     * Creates an instance of {@code JOptionPane} to display a message
     * with the specified message type, icon, and options, with the
     * initially-selected option specified.
     *
     * @param message the {@code Object} to display
     * @param messageType the type of message to be displayed:
     *                  {@code ERROR_MESSAGE},
     *                  {@code INFORMATION_MESSAGE},
     *                  {@code WARNING_MESSAGE},
     *                  {@code QUESTION_MESSAGE},
     *                  or {@code PLAIN_MESSAGE}
     * @param optionType the options to display in the pane:
     *                  {@code DEFAULT_OPTION},
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  {@code OK_CANCEL_OPTION}
     * @param icon the Icon image to display
     * @param options  the choices the user can select
     * @param initialValue the choice that is initially selected; if
     *                  {@code null}, then nothing will be initially selected;
     *                  only meaningful if {@code options} is used
     */
    public JOptionPane(Object message, int messageType, int optionType,
                       Icon icon, Object[] options, Object initialValue) {

        this.message = message;
        this.options = options;
        this.initialValue = initialValue;
        this.icon = icon;
        setMessageType(messageType);
        setOptionType(optionType);
        value = UNINITIALIZED_VALUE;
        inputValue = UNINITIALIZED_VALUE;
        updateUI();
    }

    /**
     * Sets the UI object which implements the {@literal L&F} for this component.
     *
     * @param ui  the {@code OptionPaneUI} {@literal L&F} object
     * @see UIDefaults#getUI
     * @beaninfo
     *       bound: true
     *      hidden: true
     * description: The UI object that implements the optionpane's LookAndFeel
     */
    public void setUI(OptionPaneUI ui) {
        if (this.ui != ui) {
            super.setUI(ui);
            invalidate();
        }
    }

    /**
     * Returns the UI object which implements the {@literal L&F} for this component.
     *
     * @return the {@code OptionPaneUI} object
     */
    public OptionPaneUI getUI() {
        return (OptionPaneUI)ui;
    }

    /**
     * Notification from the {@code UIManager} that the {@literal L&F} has changed.
     * Replaces the current UI object with the latest version from the
     * {@code UIManager}.
     *
     * @see JComponent#updateUI
     */
    public void updateUI() {
        setUI((OptionPaneUI)UIManager.getUI(this));
    }


    /**
     * Returns the name of the UI class that implements the
     * {@literal L&F} for this component.
     *
     * @return the string "OptionPaneUI"
     * @see JComponent#getUIClassID
     * @see UIDefaults#getUI
     */
    public String getUIClassID() {
        return uiClassID;
    }


    /**
     * Sets the option pane's message-object.
     * @param newMessage the {@code Object} to display
     * @see #getMessage
     *
     * @beaninfo
     *   preferred: true
     *   bound: true
     * description: The optionpane's message object.
     */
    public void setMessage(Object newMessage) {
        Object           oldMessage = message;

        message = newMessage;
        firePropertyChange(MESSAGE_PROPERTY, oldMessage, message);
    }

    /**
     * Returns the message-object this pane displays.
     * @see #setMessage
     *
     * @return the {@code Object} that is displayed
     */
    public Object getMessage() {
        return message;
    }

    /**
     * Sets the icon to display. If non-{@code null}, the look and feel
     * does not provide an icon.
     * @param newIcon the {@code Icon} to display
     *
     * @see #getIcon
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's type icon.
     */
    public void setIcon(Icon newIcon) {
        Object              oldIcon = icon;

        icon = newIcon;
        firePropertyChange(ICON_PROPERTY, oldIcon, icon);
    }

    /**
     * Returns the icon this pane displays.
     * @return the {@code Icon} that is displayed
     *
     * @see #setIcon
     */
    public Icon getIcon() {
        return icon;
    }

    /**
     * Sets the value the user has chosen.
     * @param newValue  the chosen value
     *
     * @see #getValue
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's value object.
     */
    public void setValue(Object newValue) {
        Object               oldValue = value;

        value = newValue;
        firePropertyChange(VALUE_PROPERTY, oldValue, value);
    }

    /**
     * Returns the value the user has selected. {@code UNINITIALIZED_VALUE}
     * implies the user has not yet made a choice, {@code null} means the
     * user closed the window with out choosing anything. Otherwise
     * the returned value will be one of the options defined in this
     * object.
     *
     * @return the {@code Object} chosen by the user,
     *         {@code UNINITIALIZED_VALUE}
     *         if the user has not yet made a choice, or {@code null} if
     *         the user closed the window without making a choice
     *
     * @see #setValue
     */
    public Object getValue() {
        return value;
    }

    /**
     * Sets the options this pane displays. If an element in
     * {@code newOptions} is a {@code Component}
     * it is added directly to the pane,
     * otherwise a button is created for the element.
     *
     * @param newOptions an array of {@code Objects} that create the
     *          buttons the user can click on, or arbitrary
     *          {@code Components} to add to the pane
     *
     * @see #getOptions
     * @beaninfo
     *       bound: true
     * description: The option pane's options objects.
     */
    public void setOptions(Object[] newOptions) {
        Object[]           oldOptions = options;

        options = newOptions;
        firePropertyChange(OPTIONS_PROPERTY, oldOptions, options);
    }

    /**
     * Returns the choices the user can make.
     * @return the array of {@code Objects} that give the user's choices
     *
     * @see #setOptions
     */
    public Object[] getOptions() {
        if(options != null) {
            int             optionCount = options.length;
            Object[]        retOptions = new Object[optionCount];

            System.arraycopy(options, 0, retOptions, 0, optionCount);
            return retOptions;
        }
        return options;
    }

    /**
     * Sets the initial value that is to be enabled -- the
     * {@code Component}
     * that has the focus when the pane is initially displayed.
     *
     * @param newInitialValue the {@code Object} that gets the initial
     *                         keyboard focus
     *
     * @see #getInitialValue
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's initial value object.
     */
    public void setInitialValue(Object newInitialValue) {
        Object            oldIV = initialValue;

        initialValue = newInitialValue;
        firePropertyChange(INITIAL_VALUE_PROPERTY, oldIV, initialValue);
    }

    /**
     * Returns the initial value.
     *
     * @return the {@code Object} that gets the initial keyboard focus
     *
     * @see #setInitialValue
     */
    public Object getInitialValue() {
        return initialValue;
    }

    /**
     * Sets the option pane's message type.
     * The message type is used by the Look and Feel to determine the
     * icon to display (if not supplied) as well as potentially how to
     * lay out the {@code parentComponent}.
     * @param newType an integer specifying the kind of message to display:
     *                {@code ERROR_MESSAGE}, {@code INFORMATION_MESSAGE},
     *                {@code WARNING_MESSAGE},
     *                {@code QUESTION_MESSAGE}, or {@code PLAIN_MESSAGE}
     * @exception RuntimeException if {@code newType} is not one of the
     *          legal values listed above

     * @see #getMessageType
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's message type.
     */
    public void setMessageType(int newType) {
        checkMessageType(newType);
        int           oldType = messageType;
        messageType = newType;
        firePropertyChange(MESSAGE_TYPE_PROPERTY, oldType, messageType);
    }

    private static void checkMessageType(int newType){
        if(newType != ERROR_MESSAGE && newType != INFORMATION_MESSAGE &&
           newType != WARNING_MESSAGE && newType != QUESTION_MESSAGE &&
           newType != PLAIN_MESSAGE)
            throw new RuntimeException("JOptionPane: type must be one of"
                    + " JOptionPane.ERROR_MESSAGE,"
                    + " JOptionPane.INFORMATION_MESSAGE,"
                    + " JOptionPane.WARNING_MESSAGE,"
                    + " JOptionPane.QUESTION_MESSAGE"
                    + " or JOptionPane.PLAIN_MESSAGE");
    }

    /**
     * Returns the message type.
     *
     * @return an integer specifying the message type
     *
     * @see #setMessageType
     */
    public int getMessageType() {
        return messageType;
    }

    /**
     * Sets the options to display.
     * The option type is used by the Look and Feel to
     * determine what buttons to show (unless options are supplied).
     * @param newType an integer specifying the options the {@literal L&F} is to display:
     *                  {@code DEFAULT_OPTION},
     *                  {@code YES_NO_OPTION},
     *                  {@code YES_NO_CANCEL_OPTION},
     *                  or {@code OK_CANCEL_OPTION}
     * @exception RuntimeException if {@code newType} is not one of
     *          the legal values listed above
     *
     * @see #getOptionType
     * @see #setOptions
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's option type.
      */
    public void setOptionType(int newType) {
        checkOptionType(newType);
        int            oldType = optionType;
        optionType = newType;
        firePropertyChange(OPTION_TYPE_PROPERTY, oldType, optionType);
    }

    private static void checkOptionType(int newType) {
        if (newType != DEFAULT_OPTION && newType != YES_NO_OPTION
                && newType != YES_NO_CANCEL_OPTION
                && newType != OK_CANCEL_OPTION) {
            throw new RuntimeException("JOptionPane: option type must be one of"
                    + " JOptionPane.DEFAULT_OPTION, JOptionPane.YES_NO_OPTION,"
                    + " JOptionPane.YES_NO_CANCEL_OPTION"
                    + " or JOptionPane.OK_CANCEL_OPTION");
        }
    }

    /**
     * Returns the type of options that are displayed.
     *
     * @return an integer specifying the user-selectable options
     *
     * @see #setOptionType
     */
    public int getOptionType() {
        return optionType;
    }

    /**
     * Sets the input selection values for a pane that provides the user
     * with a list of items to choose from. (The UI provides a widget
     * for choosing one of the values.)  A {@code null} value
     * implies the user can input whatever they wish, usually by means
     * of a {@code JTextField}.
     * <p>
     * Sets {@code wantsInput} to true. Use
     * {@code setInitialSelectionValue} to specify the initially-chosen
     * value. After the pane as been enabled, {@code inputValue} is
     * set to the value the user has selected.
     * @param newValues an array of {@code Objects} the user to be
     *                  displayed
     *                  (usually in a list or combo-box) from which
     *                  the user can make a selection
     * @see #setWantsInput
     * @see #setInitialSelectionValue
     * @see #getSelectionValues
     * @beaninfo
     *       bound: true
     * description: The option pane's selection values.
     */
    public void setSelectionValues(Object[] newValues) {
        Object[]           oldValues = selectionValues;

        selectionValues = newValues;
        firePropertyChange(SELECTION_VALUES_PROPERTY, oldValues, newValues);
        if(selectionValues != null)
            setWantsInput(true);
    }

    /**
     * Returns the input selection values.
     *
     * @return the array of {@code Objects} the user can select
     * @see #setSelectionValues
     */
    public Object[] getSelectionValues() {
        return selectionValues;
    }

    /**
     * Sets the input value that is initially displayed as selected to the user.
     * Only used if {@code wantsInput} is true.
     * @param newValue the initially selected value
     * @see #setSelectionValues
     * @see #getInitialSelectionValue
     * @beaninfo
     *       bound: true
     * description: The option pane's initial selection value object.
     */
    public void setInitialSelectionValue(Object newValue) {
        Object          oldValue = initialSelectionValue;

        initialSelectionValue = newValue;
        firePropertyChange(INITIAL_SELECTION_VALUE_PROPERTY, oldValue,
                           newValue);
    }

    /**
     * Returns the input value that is displayed as initially selected to the user.
     *
     * @return the initially selected value
     * @see #setInitialSelectionValue
     * @see #setSelectionValues
     */
    public Object getInitialSelectionValue() {
        return initialSelectionValue;
    }

    /**
     * Sets the input value that was selected or input by the user.
     * Only used if {@code wantsInput} is true.  Note that this method
     * is invoked internally by the option pane (in response to user action)
     * and should generally not be called by client programs.  To set the
     * input value initially displayed as selected to the user, use
     * {@code setInitialSelectionValue}.
     *
     * @param newValue the {@code Object} used to set the
     *          value that the user specified (usually in a text field)
     * @see #setSelectionValues
     * @see #setInitialSelectionValue
     * @see #setWantsInput
     * @see #getInputValue
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: The option pane's input value object.
     */
    public void setInputValue(Object newValue) {
        Object              oldValue = inputValue;

        inputValue = newValue;
        firePropertyChange(INPUT_VALUE_PROPERTY, oldValue, newValue);
    }

    /**
     * Returns the value the user has input, if {@code wantsInput}
     * is true.
     *
     * @return the {@code Object} the user specified,
     *          if it was one of the objects, or a
     *          {@code String} if it was a value typed into a
     *          field
     * @see #setSelectionValues
     * @see #setWantsInput
     * @see #setInputValue
     */
    public Object getInputValue() {
        return inputValue;
    }

    /**
     * Returns the maximum number of characters to place on a line in a
     * message. Default is to return {@code Integer.MAX_VALUE}.
     * The value can be
     * changed by overriding this method in a subclass.
     *
     * @return an integer giving the maximum number of characters on a line
     */
    public int getMaxCharactersPerLineCount() {
        return Integer.MAX_VALUE;
    }

    /**
     * Sets the {@code wantsInput} property.
     * If {@code newValue} is true, an input component
     * (such as a text field or combo box) whose parent is
     * {@code parentComponent} is provided to
     * allow the user to input a value. If {@code getSelectionValues}
     * returns a non-{@code null} array, the input value is one of the
     * objects in that array. Otherwise the input value is whatever
     * the user inputs.
     * <p>
     * This is a bound property.
     *
     * @param newValue if true, an input component whose parent is {@code parentComponent}
     *                 is provided to allow the user to input a value.
     * @see #setSelectionValues
     * @see #setInputValue
     * @beaninfo
     *   preferred: true
     *       bound: true
     * description: Flag which allows the user to input a value.
     */
    public void setWantsInput(boolean newValue) {
        boolean            oldValue = wantsInput;

        wantsInput = newValue;
        firePropertyChange(WANTS_INPUT_PROPERTY, oldValue, newValue);
    }

    /**
     * Returns the value of the {@code wantsInput} property.
     *
     * @return true if an input component will be provided
     * @see #setWantsInput
     */
    public boolean getWantsInput() {
        return wantsInput;
    }

    /**
     * Requests that the initial value be selected, which will set
     * focus to the initial value. This method
     * should be invoked after the window containing the option pane
     * is made visible.
     */
    public void selectInitialValue() {
        OptionPaneUI         ui = getUI();
        if (ui != null) {
            ui.selectInitialValue(this);
        }
    }


    private static int styleFromMessageType(int messageType) {
        switch (messageType) {
        case ERROR_MESSAGE:
            return JRootPane.ERROR_DIALOG;
        case QUESTION_MESSAGE:
            return JRootPane.QUESTION_DIALOG;
        case WARNING_MESSAGE:
            return JRootPane.WARNING_DIALOG;
        case INFORMATION_MESSAGE:
            return JRootPane.INFORMATION_DIALOG;
        case PLAIN_MESSAGE:
        default:
            return JRootPane.PLAIN_DIALOG;
        }
    }

    // Serialization support.
    private void writeObject(ObjectOutputStream s) throws IOException {
        Vector<Object> values = new Vector<Object>();

        s.defaultWriteObject();
        // Save the icon, if its Serializable.
        if(icon != null && icon instanceof Serializable) {
            values.addElement("icon");
            values.addElement(icon);
        }
        // Save the message, if its Serializable.
        if(message != null && message instanceof Serializable) {
            values.addElement("message");
            values.addElement(message);
        }
        // Save the treeModel, if its Serializable.
        if(options != null) {
            Vector<Object> serOptions = new Vector<Object>();

            for(int counter = 0, maxCounter = options.length;
                counter < maxCounter; counter++)
                if(options[counter] instanceof Serializable)
                    serOptions.addElement(options[counter]);
            if(serOptions.size() > 0) {
                int             optionCount = serOptions.size();
                Object[]        arrayOptions = new Object[optionCount];

                serOptions.copyInto(arrayOptions);
                values.addElement("options");
                values.addElement(arrayOptions);
            }
        }
        // Save the initialValue, if its Serializable.
        if(initialValue != null && initialValue instanceof Serializable) {
            values.addElement("initialValue");
            values.addElement(initialValue);
        }
        // Save the value, if its Serializable.
        if(value != null && value instanceof Serializable) {
            values.addElement("value");
            values.addElement(value);
        }
        // Save the selectionValues, if its Serializable.
        if(selectionValues != null) {
            boolean            serialize = true;

            for(int counter = 0, maxCounter = selectionValues.length;
                counter < maxCounter; counter++) {
                if(selectionValues[counter] != null &&
                   !(selectionValues[counter] instanceof Serializable)) {
                    serialize = false;
                    break;
                }
            }
            if(serialize) {
                values.addElement("selectionValues");
                values.addElement(selectionValues);
            }
        }
        // Save the inputValue, if its Serializable.
        if(inputValue != null && inputValue instanceof Serializable) {
            values.addElement("inputValue");
            values.addElement(inputValue);
        }
        // Save the initialSelectionValue, if its Serializable.
        if(initialSelectionValue != null &&
           initialSelectionValue instanceof Serializable) {
            values.addElement("initialSelectionValue");
            values.addElement(initialSelectionValue);
        }
        s.writeObject(values);
    }

    private void readObject(ObjectInputStream s)
        throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField f = s.readFields();

        int newMessageType = f.get("messageType", 0);
        checkMessageType(newMessageType);
        messageType = newMessageType;
        int newOptionType = f.get("optionType", 0);
        checkOptionType(newOptionType);
        optionType = newOptionType;
        wantsInput = f.get("wantsInput", false);

        Vector<?>       values = (Vector)s.readObject();
        int             indexCounter = 0;
        int             maxCounter = values.size();

        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("icon")) {
            icon = (Icon)values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("message")) {
            message = values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("options")) {
            options = (Object[])values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("initialValue")) {
            initialValue = values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("value")) {
            value = values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("selectionValues")) {
            selectionValues = (Object[])values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("inputValue")) {
            inputValue = values.elementAt(++indexCounter);
            indexCounter++;
        }
        if(indexCounter < maxCounter && values.elementAt(indexCounter).
           equals("initialSelectionValue")) {
            initialSelectionValue = values.elementAt(++indexCounter);
            indexCounter++;
        }
        if (getUIClassID().equals(uiClassID)) {
            byte count = JComponent.getWriteObjCounter(this);
            JComponent.setWriteObjCounter(this, --count);
            if (count == 0 && ui != null) {
                ui.installUI(this);
            }
        }
    }


    /**
     * Returns a string representation of this {@code JOptionPane}.
     * This method
     * is intended to be used only for debugging purposes, and the
     * content and format of the returned string may vary between
     * implementations. The returned string may be empty but may not
     * be {@code null}.
     *
     * @return  a string representation of this {@code JOptionPane}
     */
    protected String paramString() {
        String iconString = (icon != null ?
                             icon.toString() : "");
        String initialValueString = (initialValue != null ?
                                     initialValue.toString() : "");
        String messageString = (message != null ?
                                message.toString() : "");
        String messageTypeString;
        if (messageType == ERROR_MESSAGE) {
            messageTypeString = "ERROR_MESSAGE";
        } else if (messageType == INFORMATION_MESSAGE) {
            messageTypeString = "INFORMATION_MESSAGE";
        } else if (messageType == WARNING_MESSAGE) {
            messageTypeString = "WARNING_MESSAGE";
        } else if (messageType == QUESTION_MESSAGE) {
            messageTypeString = "QUESTION_MESSAGE";
        } else if (messageType == PLAIN_MESSAGE)  {
            messageTypeString = "PLAIN_MESSAGE";
        } else messageTypeString = "";
        String optionTypeString;
        if (optionType == DEFAULT_OPTION) {
            optionTypeString = "DEFAULT_OPTION";
        } else if (optionType == YES_NO_OPTION) {
            optionTypeString = "YES_NO_OPTION";
        } else if (optionType == YES_NO_CANCEL_OPTION) {
            optionTypeString = "YES_NO_CANCEL_OPTION";
        } else if (optionType == OK_CANCEL_OPTION) {
            optionTypeString = "OK_CANCEL_OPTION";
        } else optionTypeString = "";
        String wantsInputString = (wantsInput ?
                                   "true" : "false");

        return super.paramString() +
        ",icon=" + iconString +
        ",initialValue=" + initialValueString +
        ",message=" + messageString +
        ",messageType=" + messageTypeString +
        ",optionType=" + optionTypeString +
        ",wantsInput=" + wantsInputString;
    }

///////////////////
// Accessibility support
///////////////////

    /**
     * Returns the {@code AccessibleContext} associated with this JOptionPane.
     * For option panes, the {@code AccessibleContext} takes the form of an
     * {@code AccessibleJOptionPane}.
     * A new {@code AccessibleJOptionPane} instance is created if necessary.
     *
     * @return an AccessibleJOptionPane that serves as the
     *         AccessibleContext of this AccessibleJOptionPane
     * @beaninfo
     *       expert: true
     *  description: The AccessibleContext associated with this option pane
     */
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleJOptionPane();
        }
        return accessibleContext;
    }

    /**
     * This class implements accessibility support for the
     * {@code JOptionPane} class.  It provides an implementation of the
     * Java Accessibility API appropriate to option pane user-interface
     * elements.
     * <p>
     * <strong>Warning:</strong>
     * Serialized objects of this class will not be compatible with
     * future Swing releases. The current serialization support is
     * appropriate for short term storage or RMI between applications running
     * the same version of Swing.  As of 1.4, support for long term storage
     * of all JavaBeans&trade;
     * has been added to the {@code java.beans} package.
     * Please see {@link java.beans.XMLEncoder}.
     */
    @SuppressWarnings("serial") // Same-version serialization only
    protected class AccessibleJOptionPane extends AccessibleJComponent {

        /**
         * Get the role of this object.
         *
         * @return an instance of AccessibleRole describing the role of the object
         * @see AccessibleRole
         */
        public AccessibleRole getAccessibleRole() {
            switch (messageType) {
            case ERROR_MESSAGE:
            case INFORMATION_MESSAGE:
            case WARNING_MESSAGE:
                return AccessibleRole.ALERT;

            default:
                return AccessibleRole.OPTION_PANE;
            }
        }

    } // inner class AccessibleJOptionPane
}
