/*
 * Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package javax.swing;

import java.util.*;
import java.io.Serializable;


/**
 * A {@code SpinnerModel} for sequences of {@code Date}s.
 * The upper and lower bounds of the sequence are defined by properties called
 * {@code start} and {@code end} and the size
 * of the increase or decrease computed by the {@code nextValue}
 * and {@code previousValue} methods is defined by a property
 * called {@code calendarField}.  The {@code start}
 * and {@code end} properties can be {@code null} to
 * indicate that the sequence has no lower or upper limit.
 * <p>
 * The value of the {@code calendarField} property must be one of the
 * {@code java.util.Calendar} constants that specify a field
 * within a {@code Calendar}.  The {@code getNextValue}
 * and {@code getPreviousValue}
 * methods change the date forward or backwards by this amount.
 * For example, if {@code calendarField} is {@code Calendar.DAY_OF_WEEK},
 * then {@code nextValue} produces a {@code Date} that's 24
 * hours after the current {@code value}, and {@code previousValue}
 * produces a {@code Date} that's 24 hours earlier.
 * <p>
 * The legal values for {@code calendarField} are:
 * <ul>
 *   <li>{@code Calendar.ERA}
 *   <li>{@code Calendar.YEAR}
 *   <li>{@code Calendar.MONTH}
 *   <li>{@code Calendar.WEEK_OF_YEAR}
 *   <li>{@code Calendar.WEEK_OF_MONTH}
 *   <li>{@code Calendar.DAY_OF_MONTH}
 *   <li>{@code Calendar.DAY_OF_YEAR}
 *   <li>{@code Calendar.DAY_OF_WEEK}
 *   <li>{@code Calendar.DAY_OF_WEEK_IN_MONTH}
 *   <li>{@code Calendar.AM_PM}
 *   <li>{@code Calendar.HOUR}
 *   <li>{@code Calendar.HOUR_OF_DAY}
 *   <li>{@code Calendar.MINUTE}
 *   <li>{@code Calendar.SECOND}
 *   <li>{@code Calendar.MILLISECOND}
 * </ul>
 * However some UIs may set the calendarField before committing the edit
 * to spin the field under the cursor. If you only want one field to
 * spin you can subclass and ignore the setCalendarField calls.
 * <p>
 * This model inherits a {@code ChangeListener}.  The
 * {@code ChangeListeners} are notified whenever the models
 * {@code value}, {@code calendarField},
 * {@code start}, or {@code end} properties changes.
 *
 * @see JSpinner
 * @see SpinnerModel
 * @see AbstractSpinnerModel
 * @see SpinnerListModel
 * @see SpinnerNumberModel
 * @see Calendar#add
 *
 * @author Hans Muller
 * @since 1.4
 */
@SuppressWarnings("serial") // Superclass is not serializable across versions
public class SpinnerDateModel extends AbstractSpinnerModel implements Serializable
{
    private Comparable<Date> start, end;
    private Calendar value;
    private int calendarField;


    private boolean calendarFieldOK(int calendarField) {
        switch(calendarField) {
        case Calendar.ERA:
        case Calendar.YEAR:
        case Calendar.MONTH:
        case Calendar.WEEK_OF_YEAR:
        case Calendar.WEEK_OF_MONTH:
        case Calendar.DAY_OF_MONTH:
        case Calendar.DAY_OF_YEAR:
        case Calendar.DAY_OF_WEEK:
        case Calendar.DAY_OF_WEEK_IN_MONTH:
        case Calendar.AM_PM:
        case Calendar.HOUR:
        case Calendar.HOUR_OF_DAY:
        case Calendar.MINUTE:
        case Calendar.SECOND:
        case Calendar.MILLISECOND:
            return true;
        default:
            return false;
        }
    }


    /**
     * Creates a {@code SpinnerDateModel} that represents a sequence of dates
     * between {@code start} and {@code end}.  The
     * {@code nextValue} and {@code previousValue} methods
     * compute elements of the sequence by advancing or reversing
     * the current date {@code value} by the
     * {@code calendarField} time unit.  For a precise description
     * of what it means to increment or decrement a {@code Calendar}
     * {@code field}, see the {@code add} method in
     * {@code java.util.Calendar}.
     * <p>
     * The {@code start} and {@code end} parameters can be
     * {@code null} to indicate that the range doesn't have an
     * upper or lower bound.  If {@code value} or
     * {@code calendarField} is {@code null}, or if both
     * {@code start} and {@code end} are specified and
     * {@code minimum > maximum} then an
     * {@code IllegalArgumentException} is thrown.
     * Similarly if {@code (minimum <= value <= maximum)} is false,
     * an IllegalArgumentException is thrown.
     *
     * @param value the current (non {@code null}) value of the model
     * @param start the first date in the sequence or {@code null}
     * @param end the last date in the sequence or {@code null}
     * @param calendarField one of
     *   <ul>
     *    <li>{@code Calendar.ERA}
     *    <li>{@code Calendar.YEAR}
     *    <li>{@code Calendar.MONTH}
     *    <li>{@code Calendar.WEEK_OF_YEAR}
     *    <li>{@code Calendar.WEEK_OF_MONTH}
     *    <li>{@code Calendar.DAY_OF_MONTH}
     *    <li>{@code Calendar.DAY_OF_YEAR}
     *    <li>{@code Calendar.DAY_OF_WEEK}
     *    <li>{@code Calendar.DAY_OF_WEEK_IN_MONTH}
     *    <li>{@code Calendar.AM_PM}
     *    <li>{@code Calendar.HOUR}
     *    <li>{@code Calendar.HOUR_OF_DAY}
     *    <li>{@code Calendar.MINUTE}
     *    <li>{@code Calendar.SECOND}
     *    <li>{@code Calendar.MILLISECOND}
     *   </ul>
     *
     * @throws IllegalArgumentException if {@code value} or
     *    {@code calendarField} are {@code null},
     *    if {@code calendarField} isn't valid,
     *    or if the following expression is
     *    false: {@code (start <= value <= end)}.
     *
     * @see Calendar#add
     * @see #setValue
     * @see #setStart
     * @see #setEnd
     * @see #setCalendarField
     */
    public SpinnerDateModel(Date value, Comparable<Date> start, Comparable<Date> end, int calendarField) {
        if (value == null) {
            throw new IllegalArgumentException("value is null");
        }
        if (!calendarFieldOK(calendarField)) {
            throw new IllegalArgumentException("invalid calendarField");
        }
        if (!(((start == null) || (start.compareTo(value) <= 0)) &&
              ((end == null) || (end.compareTo(value) >= 0)))) {
            throw new IllegalArgumentException("(start <= value <= end) is false");
        }
        this.value = Calendar.getInstance();
        this.start = start;
        this.end = end;
        this.calendarField = calendarField;

        this.value.setTime(value);
    }


    /**
     * Constructs a {@code SpinnerDateModel} whose initial
     * {@code value} is the current date, {@code calendarField}
     * is equal to {@code Calendar.DAY_OF_MONTH}, and for which
     * there are no {@code start}/{@code end} limits.
     */
    public SpinnerDateModel() {
        this(new Date(), null, null, Calendar.DAY_OF_MONTH);
    }


    /**
     * Changes the lower limit for Dates in this sequence.
     * If {@code start} is {@code null},
     * then there is no lower limit.  No bounds checking is done here:
     * the new start value may invalidate the
     * {@code (start <= value <= end)}
     * invariant enforced by the constructors.  This is to simplify updating
     * the model.  Naturally one should ensure that the invariant is true
     * before calling the {@code nextValue}, {@code previousValue},
     * or {@code setValue} methods.
     * <p>
     * Typically this property is a {@code Date} however it's possible to use
     * a {@code Comparable} with a {@code compareTo} method for Dates.
     * For example {@code start} might be an instance of a class like this:
     * <pre>
     * MyStartDate implements Comparable {
     *     long t = 12345;
     *     public int compareTo(Date d) {
     *            return (t &lt; d.getTime() ? -1 : (t == d.getTime() ? 0 : 1));
     *     }
     *     public int compareTo(Object o) {
     *            return compareTo((Date)o);
     *     }
     * }
     * </pre>
     * Note that the above example will throw a {@code ClassCastException}
     * if the {@code Object} passed to {@code compareTo(Object)}
     * is not a {@code Date}.
     * <p>
     * This method fires a {@code ChangeEvent} if the
     * {@code start} has changed.
     *
     * @param start defines the first date in the sequence
     * @see #getStart
     * @see #setEnd
     * @see #addChangeListener
     */
    public void setStart(Comparable<Date> start) {
        if ((start == null) ? (this.start != null) : !start.equals(this.start)) {
            this.start = start;
            fireStateChanged();
        }
    }


    /**
     * Returns the first {@code Date} in the sequence.
     *
     * @return the value of the {@code start} property
     * @see #setStart
     */
    public Comparable<Date> getStart() {
        return start;
    }


    /**
     * Changes the upper limit for {@code Date}s in this sequence.
     * If {@code start} is {@code null}, then there is no upper
     * limit.  No bounds checking is done here: the new
     * start value may invalidate the {@code (start <= value <= end)}
     * invariant enforced by the constructors.  This is to simplify updating
     * the model.  Naturally, one should ensure that the invariant is true
     * before calling the {@code nextValue}, {@code previousValue},
     * or {@code setValue} methods.
     * <p>
     * Typically this property is a {@code Date} however it's possible to use
     * {@code Comparable} with a {@code compareTo} method for
     * {@code Date}s.  See {@code setStart} for an example.
     * <p>
     * This method fires a {@code ChangeEvent} if the {@code end}
     * has changed.
     *
     * @param end defines the last date in the sequence
     * @see #getEnd
     * @see #setStart
     * @see #addChangeListener
     */
    public void setEnd(Comparable<Date> end) {
        if ((end == null) ? (this.end != null) : !end.equals(this.end)) {
            this.end = end;
            fireStateChanged();
        }
    }


    /**
     * Returns the last {@code Date} in the sequence.
     *
     * @return the value of the {@code end} property
     * @see #setEnd
     */
    public Comparable<Date> getEnd() {
        return end;
    }


    /**
     * Changes the size of the date value change computed
     * by the {@code nextValue} and {@code previousValue} methods.
     * The {@code calendarField} parameter must be one of the
     * {@code Calendar} field constants like {@code Calendar.MONTH}
     * or {@code Calendar.MINUTE}.
     * The {@code nextValue} and {@code previousValue} methods
     * simply move the specified {@code Calendar} field forward or backward
     * by one unit with the {@code Calendar.add} method.
     * You should use this method with care as some UIs may set the
     * calendarField before committing the edit to spin the field under
     * the cursor. If you only want one field to spin you can subclass
     * and ignore the setCalendarField calls.
     *
     * @param calendarField one of
     *  <ul>
     *    <li>{@code Calendar.ERA}
     *    <li>{@code Calendar.YEAR}
     *    <li>{@code Calendar.MONTH}
     *    <li>{@code Calendar.WEEK_OF_YEAR}
     *    <li>{@code Calendar.WEEK_OF_MONTH}
     *    <li>{@code Calendar.DAY_OF_MONTH}
     *    <li>{@code Calendar.DAY_OF_YEAR}
     *    <li>{@code Calendar.DAY_OF_WEEK}
     *    <li>{@code Calendar.DAY_OF_WEEK_IN_MONTH}
     *    <li>{@code Calendar.AM_PM}
     *    <li>{@code Calendar.HOUR}
     *    <li>{@code Calendar.HOUR_OF_DAY}
     *    <li>{@code Calendar.MINUTE}
     *    <li>{@code Calendar.SECOND}
     *    <li>{@code Calendar.MILLISECOND}
     *  </ul>
     * <p>
     * This method fires a {@code ChangeEvent} if the
     * {@code calendarField} has changed.
     *
     * @see #getCalendarField
     * @see #getNextValue
     * @see #getPreviousValue
     * @see Calendar#add
     * @see #addChangeListener
     */
    public void setCalendarField(int calendarField) {
        if (!calendarFieldOK(calendarField)) {
            throw new IllegalArgumentException("invalid calendarField");
        }
        if (calendarField != this.calendarField) {
            this.calendarField = calendarField;
            fireStateChanged();
        }
    }


    /**
     * Returns the {@code Calendar} field that is added to or subtracted from
     * by the {@code nextValue} and {@code previousValue} methods.
     *
     * @return the value of the {@code calendarField} property
     * @see #setCalendarField
     */
    public int getCalendarField() {
        return calendarField;
    }


    /**
     * Returns the next {@code Date} in the sequence, or {@code null} if
     * the next date is after {@code end}.
     *
     * @return the next {@code Date} in the sequence, or {@code null} if
     *     the next date is after {@code end}.
     *
     * @see SpinnerModel#getNextValue
     * @see #getPreviousValue
     * @see #setCalendarField
     */
    public Object getNextValue() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(value.getTime());
        cal.add(calendarField, 1);
        Date next = cal.getTime();
        return ((end == null) || (end.compareTo(next) >= 0)) ? next : null;
    }


    /**
     * Returns the previous {@code Date} in the sequence, or {@code null}
     * if the previous date is before {@code start}.
     *
     * @return the previous {@code Date} in the sequence, or
     *     {@code null} if the previous date
     *     is before {@code start}
     *
     * @see SpinnerModel#getPreviousValue
     * @see #getNextValue
     * @see #setCalendarField
     */
    public Object getPreviousValue() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(value.getTime());
        cal.add(calendarField, -1);
        Date prev = cal.getTime();
        return ((start == null) || (start.compareTo(prev) <= 0)) ? prev : null;
    }


    /**
     * Returns the current element in this sequence of {@code Date}s.
     * This method is equivalent to {@code (Date)getValue}.
     *
     * @return the {@code value} property
     * @see #setValue
     */
    public Date getDate() {
        return value.getTime();
    }


    /**
     * Returns the current element in this sequence of {@code Date}s.
     *
     * @return the {@code value} property
     * @see #setValue
     * @see #getDate
     */
    public Object getValue() {
        return value.getTime();
    }


    /**
     * Sets the current {@code Date} for this sequence.
     * If {@code value} is {@code null},
     * an {@code IllegalArgumentException} is thrown.  No bounds
     * checking is done here:
     * the new value may invalidate the {@code (start <= value < end)}
     * invariant enforced by the constructors.  Naturally, one should ensure
     * that the {@code (start <= value <= maximum)} invariant is true
     * before calling the {@code nextValue}, {@code previousValue},
     * or {@code setValue} methods.
     * <p>
     * This method fires a {@code ChangeEvent} if the
     * {@code value} has changed.
     *
     * @param value the current (non {@code null})
     *    {@code Date} for this sequence
     * @throws IllegalArgumentException if value is {@code null}
     *    or not a {@code Date}
     * @see #getDate
     * @see #getValue
     * @see #addChangeListener
     */
    public void setValue(Object value) {
        if ((value == null) || !(value instanceof Date)) {
            throw new IllegalArgumentException("illegal value");
        }
        if (!value.equals(this.value.getTime())) {
            this.value.setTime((Date)value);
            fireStateChanged();
        }
    }
}
