/*
 * Copyright (c) 1997, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package javax.swing.tree;

import javax.swing.event.*;

/**
 * The model used by {@code JTree}.
 * <p>
 * {@code JTree} and its related classes make extensive use of
 * {@code TreePath}s for identifying nodes in the {@code TreeModel}.
 * If a {@code TreeModel} returns the same object, as compared by
 * {@code equals}, at two different indices under the same parent
 * than the resulting {@code TreePath} objects will be considered equal
 * as well. Some implementations may assume that if two
 * {@code TreePath}s are equal, they identify the same node. If this
 * condition is not met, painting problems and other oddities may result.
 * In other words, if {@code getChild} for a given parent returns
 * the same Object (as determined by {@code equals}) problems may
 * result, and it is recommended you avoid doing this.
 * <p>
 * Similarly {@code JTree} and its related classes place
 * {@code TreePath}s in {@code Map}s.  As such if
 * a node is requested twice, the return values must be equal
 * (using the {@code equals} method) and have the same
 * {@code hashCode}.
 * <p>
 * For further information on tree models,
 * including an example of a custom implementation,
 * see <a
 href="http://docs.oracle.com/javase/tutorial/uiswing/components/tree.html">How to Use Trees</a>
 * in <em>The Java Tutorial.</em>
 *
 * @see TreePath
 *
 * @author Rob Davis
 * @author Ray Ryan
 */
public interface TreeModel
{

    /**
     * Returns the root of the tree.  Returns {@code null}
     * only if the tree has no nodes.
     *
     * @return  the root of the tree
     */
    public Object getRoot();


    /**
     * Returns the child of {@code parent} at index {@code index}
     * in the parent's
     * child array.  {@code parent} must be a node previously obtained
     * from this data source. This should not return {@code null}
     * if {@code index}
     * is a valid index for {@code parent} (that is
     * {@code index >= 0 && index < getChildCount(parent)}).
     *
     * @param parent    a node in the tree, obtained from this data source
     * @param index     index of child to be returned
     * @return          the child of {@code parent} at index {@code index}
     */
    public Object getChild(Object parent, int index);


    /**
     * Returns the number of children of {@code parent}.
     * Returns 0 if the node
     * is a leaf or if it has no children.  {@code parent} must be a node
     * previously obtained from this data source.
     *
     * @param   parent  a node in the tree, obtained from this data source
     * @return  the number of children of the node {@code parent}
     */
    public int getChildCount(Object parent);


    /**
     * Returns {@code true} if {@code node} is a leaf.
     * It is possible for this method to return {@code false}
     * even if {@code node} has no children.
     * A directory in a filesystem, for example,
     * may contain no files; the node representing
     * the directory is not a leaf, but it also has no children.
     *
     * @param   node  a node in the tree, obtained from this data source
     * @return  true if {@code node} is a leaf
     */
    public boolean isLeaf(Object node);

    /**
      * Messaged when the user has altered the value for the item identified
      * by {@code path} to {@code newValue}.
      * If {@code newValue} signifies a truly new value
      * the model should post a {@code treeNodesChanged} event.
      *
      * @param path path to the node that the user has altered
      * @param newValue the new value from the TreeCellEditor
      */
    public void valueForPathChanged(TreePath path, Object newValue);

    /**
     * Returns the index of child in parent.  If either {@code parent}
     * or {@code child} is {@code null}, returns -1.
     * If either {@code parent} or {@code child} don't
     * belong to this tree model, returns -1.
     *
     * @param parent a node in the tree, obtained from this data source
     * @param child the node we are interested in
     * @return the index of the child in the parent, or -1 if either
     *    {@code child} or {@code parent} are {@code null}
     *    or don't belong to this tree model
     */
    public int getIndexOfChild(Object parent, Object child);

//
//  Change Events
//

    /**
     * Adds a listener for the {@code TreeModelEvent}
     * posted after the tree changes.
     *
     * @param   l       the listener to add
     * @see     #removeTreeModelListener
     */
    void addTreeModelListener(TreeModelListener l);

    /**
     * Removes a listener previously added with
     * {@code addTreeModelListener}.
     *
     * @see     #addTreeModelListener
     * @param   l       the listener to remove
     */
    void removeTreeModelListener(TreeModelListener l);

}
